import math 



#==========================
# Help functions
#==========================

def grid_size(grid_elements):
    return len(grid_elements)

def sub_size(grid_elements):
    return int(math.sqrt(grid_size(grid_elements)))

def show_grid( grid, grid_elements ):
    for i in range(grid_size(grid_elements)):
        for j in range(grid_size(grid_elements)):
            print( grid[i][j], end="" )
        print()
    print()

def next_grid_pos( row, col, grid_elements ):
    gsize = grid_size(grid_elements)
    row = ( row + (col + 1)//gsize )%gsize
    col = ( col + 1 )%gsize
    return (row, col)

def solve_soduko_interface( grid_data ):
    (grid_elts, grid) = grid_data
    print("Input grid:")
    show_grid( grid, grid_elts )
    dummy = input() # Press any key to continue...
    solve_sudoku( grid, grid_elts )
    print("Solved grid:")
    show_grid( grid, grid_elts )



#==========================
# Main functions
#==========================

def valid_values( grid, row, col, grid_elements ):
    if grid[row][col] != 0:
        res = [ grid[row][col] ]
    else:
        res = grid_elements[:]
        gsize = grid_size(grid_elements)
        subsize = sub_size(grid_elements)
        for i in range(gsize):
            if grid[row][i] in res:
                res.remove(grid[row][i])
            if grid[i][col] in res:
                res.remove(grid[i][col])
            sub_row = (row//subsize)*subsize + i//subsize
            sub_col = (col//subsize)*subsize + i%subsize
            if grid[sub_row][sub_col] in res:
                res.remove(grid[sub_row][sub_col])
    return res
            
def recursive_solve_sudoku( grid, row, col, grid_elements ):
    gsize = grid_size(grid_elements)
    original_value = grid[row][col]
    possible_values = valid_values( grid, row, col, grid_elements )
    res = False
    if len(possible_values) > 0:
        if (row,col) == (gsize-1,gsize-1):
            grid[row][col] = possible_values[0]
            res = True
        else:
            (next_row,next_col) = next_grid_pos( row, col, grid_elements )
            i = 0
            while not res and i < len(possible_values):
                grid[row][col] = possible_values[i]
                res = recursive_solve_sudoku( grid, next_row, next_col, grid_elements )
                i += 1
            if not res:
                grid[row][col] = original_value                
    return res

def solve_sudoku( grid, grid_elements ):
    return recursive_solve_sudoku( grid, 0, 0, grid_elements )



#==========================
# Example grids
#==========================

def example_grid_1():
    return ( list(range(1,10)),
             [  [  5,3,0,  0,7,0,  0,0,0  ], \
                [  6,0,0,  1,9,5,  0,0,0  ], \
                [  0,9,8,  0,0,0,  0,6,0  ], \
                
                [  8,0,0,  0,6,0,  0,0,3  ], \
                [  4,0,0,  8,0,3,  0,0,1  ], \
                [  7,0,0,  0,2,0,  0,0,6  ], \
                
                [  0,6,0,  0,0,0,  2,8,0  ], \
                [  0,0,0,  4,1,9,  0,0,5  ], \
                [  0,0,0,  0,8,0,  0,7,9  ] ] )

def example_grid_2():
    return ( list(range(1,10)),
             [  [  9,0,7,  6,0,0,  2,0,3  ], \
                [  0,0,0,  3,5,0,  6,0,0  ], \
                [  0,3,0,  0,0,0,  4,0,0  ], \
                
                [  0,0,0,  2,0,0,  8,3,7  ], \
                [  0,0,0,  7,0,5,  0,0,0  ], \
                [  7,6,1,  0,0,8,  0,0,0  ], \
                
                [  0,0,8,  0,0,0,  0,5,0  ], \
                [  0,0,5,  0,7,1,  0,0,0  ], \
                [  4,0,6,  0,0,3,  1,0,8  ] ] )

def example_grid_3():
    return ( list(range(1,10)),
             [ [  0,0,0,  9,0,0,  7,2,8  ],
               [  2,7,8,  0,0,3,  0,1,0  ],
               [  0,9,0,  0,0,0,  6,4,0  ],

               [  0,5,0,  0,6,0,  2,0,0  ],
               [  0,0,6,  0,0,0,  3,0,0  ],
               [  0,1,0,  0,5,0,  0,0,0  ],

               [  1,0,0,  7,0,6,  0,3,4  ],
               [  0,0,0,  5,0,4,  0,0,0  ],
               [  7,0,9,  1,0,0,  8,0,5  ] ] )

def example_grid_3_alpha():
    return ( list(map(lambda x:chr(x), range(65,65+9))),
             [ [   0 , 0 , 0 ,  'I', 0 , 0 ,  'G','B','H'  ],
               [  'B','G','H',   0 , 0 ,'C',   0 ,'A', 0   ],
               [   0 ,'I', 0 ,   0 , 0 , 0 ,  'F','D', 0   ],

               [   0 ,'E', 0 ,   0 ,'F', 0 ,  'B', 0 , 0   ],
               [   0 , 0 ,'F',   0 , 0 , 0 ,  'C', 0 , 0   ],
               [   0 ,'A', 0 ,   0 ,'E', 0 ,   0 , 0 , 0   ],

               [  'A', 0 , 0 ,  'G', 0 ,'F',   0 ,'C','D'  ],
               [   0 , 0 , 0 ,  'E', 0 ,'D',   0 , 0 , 0   ],
               [  'G', 0 ,'I',  'A', 0 , 0 ,  'H', 0 ,'E'  ] ])    

def example_grid_4_alpha():
    return ( list(map(lambda x:chr(x), range(65,65+25))),
             [ [  'E',0,0,'X',0,  0,'A',0,0,'B',  0,0,'R','J',0,  0,'K',0,'O',0,  'P',0,0,0,0  ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],

               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
             
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
             
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
             
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ],
               [  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0,  0,0,0,0,0, ] ] )



#==========================
# Example execution
#==========================

solve_soduko_interface( example_grid_1() )
